package net.sf.javaprinciples.resource;

public abstract class AbstractResource<T> implements Resource<T>
{
    protected long validFrom;
    protected long validUntil;
    protected String mimeType;
    protected String encoding;
    protected T content;
    
    /**
     * A resource may be valid from a point in time.
     * This typically will be the time the resource was created. It may not be the
     * time the resource was first served.
     * @return A time in milliseconds since 1970 relative to UTC.
     */
    @Override
    public long getValidFrom()
    {
        return validFrom;
    }
    
    /**
     * A resource may be valid only up to a point in time.
     * This typically will be the time the resource may be updated.
     * @return A time in milliseconds since 1970 relative to UTC.
     */
    @Override
    public long getValidUntil()
    {
        return validUntil;
    }

    /**
     * Each resource holds content of a specific type.
     * @return The mime type of the content.  
     */
    @Override
    public String getMimeType()
    {
        return mimeType;
    }

    /**
     * The encoding for the resource.
     * @return A standard encoding may be null
     */
    @Override
    public String getEncoding()
    {
        return encoding;
    }

    /**
     * @param content The content of the resource.
     */
    @Override
    public void setContent(T content)
    {
        this.content = content;
    }
    /**
     * {@inheritDoc}
     */
    @Override
    public T getContent()
    {
        return content;
    }
    
    /**
     * Lifespan of the Resource in milliseconds.
     * If it is not set, or set to zero or a negative number, the Resource will not expire.
     * @param amount    milliseconds, positive number applies an expiry time,
     * zero or negative means the Resource will not expire.
     */
    public void setLifespan(long amount)
    {
        this.validFrom = System.currentTimeMillis();

        if (amount > 0)
        {
            this.validUntil = this.validFrom + amount;
        }
        else
        {
            this.validUntil = 0;
        }
    }
    
}
