package net.sf.javaprinciples.resource.servlet;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import net.sf.javaprinciples.core.SystemException;
import net.sf.javaprinciples.resource.ResourceErrors;
import net.sf.javaprinciples.resource.ResourceException;
import net.sf.javaprinciples.resource.ResourceNotFoundException;
import net.sf.javaprinciples.resource.ResourceParseException;

@Controller
public final class ResourceController<T>
{
    private static final Log LOGGER = LogFactory.getLog(ResourceController.class);
    private static final String APPLICATION_ERROR_CODE = "X-Application-Error-Code";

    private RequestHandler<T> getHandler;
    private RequestHandler<T> putHandler;
    private RequestHandler<T> postHandler;
    private RequestHandler<T> deleteHandler;
    private Map<ResourceErrors, String> errorMap;

    /**
     * Handle a GET request.
     * @param model the model propagated by Spring through MVC infrastructure.
     * @param request Never null
     */
    @RequestMapping(method = {RequestMethod.GET})
    public void handleGetRequest(Model model, HttpServletRequest request)
    {
        handleRequestInternal(model, request, getHandler);
    }

    /**
     * Handle a PUT request.
     * @param model the model propagated by Spring through MVC infrastructure.
     * @param request Never null
     */
    @RequestMapping(method = {RequestMethod.PUT})
    public void handlePutRequest(Model model, HttpServletRequest request)
    {
        handleRequestInternal(model, request, putHandler);
    }

    /**
     * Handle a POST request.
     * @param model the model propagated by Spring through MVC infrastructure.
     * @param request Never null
     */
    @RequestMapping(method = {RequestMethod.POST})
    public void handlePostRequest(Model model, HttpServletRequest request)
    {
        handleRequestInternal(model, request, postHandler);
    }

    /**
     * Handle a DELETE request.
     * @param model the model propagated by Spring through MVC infrastructure.
     * @param request Never null
     */
    @RequestMapping(method = {RequestMethod.DELETE})
    public void handleDeleteRequest(Model model, HttpServletRequest request)
    {
        handleRequestInternal(model, request, deleteHandler);
    }

    private void handleRequestInternal(Model model, HttpServletRequest request, RequestHandler<T> handler)
    {
        ResourceModel<T> resourceModel = prepareModel(model);

        try
        {
            handler.handleRequest(resourceModel, request);
        }
        // Additional exceptions handled depending on error condition.
        catch (ResourceParseException rse)
        {
            resourceModel.setStatusCode(HttpServletResponse.SC_BAD_REQUEST);
        }
        catch (ResourceNotFoundException re)
        {
            resourceModel.setStatusCode(HttpServletResponse.SC_NO_CONTENT);
        }
        catch (ResourceException re)
        {
            resourceModel.setStatusCode(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            resourceModel.addProperty(APPLICATION_ERROR_CODE, resolveErrorCode(re));
            LOGGER.error(re.getMessage());
        }
        catch (SystemException e)
        {
            resourceModel.setStatusCode(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            resourceModel.addProperty(APPLICATION_ERROR_CODE, resolveErrorCode(e));
            LOGGER.error(e.getMessage());
        }
    }

    private String resolveErrorCode(Exception e)
    {
        String message = e.getMessage();
        if (errorMap == null)
        {
            return message;
        }
        return errorMap.get(ResourceErrors.valueOf(message));
    }

    private ResourceModel<T> prepareModel(Model model)
    {
        ResourceModel<T> resourceModel = new ResourceModel<T>();
        model.addAttribute(ResourceModel.class.getName(), resourceModel);
        return resourceModel;
    }

    public void setGetHandler(RequestHandler<T> getHandler)
    {
        this.getHandler = getHandler;
    }

    public void setPutHandler(RequestHandler<T> putHandler)
    {
        this.putHandler = putHandler;
    }

    public void setPostHandler(RequestHandler<T> postHandler)
    {
        this.postHandler = postHandler;
    }

    public void setDeleteHandler(RequestHandler<T> deleteHandler)
    {
        this.deleteHandler = deleteHandler;
    }

    public void setErrorMap(Map<ResourceErrors, String> errorMap)
    {
        this.errorMap = errorMap;
    }
}